@@ -1,3 +1,5 @@
+require 'rspec/core'  # to fix annoying "undefined method `configuration' for RSpec:Module (NoMethodError)"
+
 Then /^the output should contain all of these:$/ do |table|
   table.raw.flatten.each do |string|
     assert_partial_output(string, all_output)
@@ -10,13 +12,23 @@ Then /^the output should not contain any of these:$/ do |table|
   end
 end
 
+Then /^the output should contain one of the following:$/ do |table|
+  matching_output = table.raw.flatten.select do |string|
+    all_output =~ regexp(string)
+  end
+
+  matching_output.should have(1).item
+end
+
 Then /^the example(?:s)? should(?: all)? pass$/ do
   step %q{the output should contain "0 failures"}
+  step %q{the output should not contain "0 examples"}
   step %q{the exit status should be 0}
 end
 
-Then /^the file "([^"]*)" should contain:$/ do |file, partial_content|
-  check_file_content(file, partial_content, true)
+Then /^the process should succeed even though no examples were run$/ do
+  step %q{the output should contain "0 examples, 0 failures"}
+  step %q{the exit status should be 0}
 end
 
 Then /^the backtrace\-normalized output should contain:$/ do |partial_output|
@@ -28,3 +40,10 @@ Then /^the backtrace\-normalized output should contain:$/ do |partial_output|
 
   normalized_output.should =~ regexp(partial_output)
 end
+
+# This step can be generalized if it's ever used to test other colors
+Then /^the failing example is printed in magenta$/ do
+  # \e[35m = enable magenta
+  # \e[0m  = reset colors
+  expect(all_output).to include("\e[35m" + "F" + "\e[0m")
+end
