@@ -1,7 +1,7 @@
 module RSpec
   module Core
-    # ExampleGroup and Example are the main structural elements of rspec-core.
-    # Consider this example:
+    # ExampleGroup and {Example} are the main structural elements of
+    # rspec-core.  Consider this example:
     #
     #     describe Thing do
     #       it "does something" do
@@ -15,13 +15,13 @@ module RSpec
     class ExampleGroup
       extend  MetadataHashBuilder::WithDeprecationWarning
       extend  Extensions::ModuleEvalWithArgs
-      extend  Subject::ExampleGroupMethods
       extend  Hooks
 
+      include MemoizedHelpers
       include Extensions::InstanceEvalWithArgs
-      include Subject::ExampleMethods
       include Pending
-      include Let
+      include SharedExampleGroup
+      extend SharedExampleGroup
 
       # @private
       def self.world
@@ -43,85 +43,130 @@ module RSpec
           end
         end
 
-        delegate_to_metadata :description, :described_class, :file_path
+        def description
+          description = metadata[:example_group][:description]
+          RSpec.configuration.format_docstrings_block.call(description)
+        end
+
+        delegate_to_metadata :described_class, :file_path
         alias_method :display_name, :description
         # @private
         alias_method :describes, :described_class
-      end
 
-      # @private
-      def self.define_example_method(name, extra_options={})
-        module_eval(<<-END_RUBY, __FILE__, __LINE__)
-          def self.#{name}(desc=nil, *args, &block)
-            options = build_metadata_hash_from(args)
-            options.update(:pending => RSpec::Core::Pending::NOT_YET_IMPLEMENTED) unless block
-            options.update(#{extra_options.inspect})
-            examples << RSpec::Core::Example.new(self, desc, options, block)
-            examples.last
-          end
-        END_RUBY
-      end
-
-      define_example_method :example
-      define_example_method :it
-      define_example_method :specify
-
-      define_example_method :focused,  :focused => true, :focus => true
-      define_example_method :focus,    :focused => true, :focus => true
-
-      define_example_method :pending,  :pending => true
-      define_example_method :xexample, :pending => 'Temporarily disabled with xexample'
-      define_example_method :xit,      :pending => 'Temporarily disabled with xit'
-      define_example_method :xspecify, :pending => 'Temporarily disabled with xspecify'
-
-      class << self
-        alias_method :alias_example_to, :define_example_method
-      end
-
-      # @private
-      def self.define_nested_shared_group_method(new_name, report_label=nil)
-        module_eval(<<-END_RUBY, __FILE__, __LINE__)
-          def self.#{new_name}(name, *args, &customization_block)
-            group = describe("#{report_label || "it should behave like"} \#{name}") do
-              find_and_eval_shared("examples", name, *args, &customization_block)
+        # @private
+        # @macro [attach] define_example_method
+        #   @param [String] name
+        #   @param [Hash] extra_options
+        #   @param [Block] implementation
+        def self.define_example_method(name, extra_options={})
+          module_eval(<<-END_RUBY, __FILE__, __LINE__)
+            def #{name}(desc=nil, *args, &block)
+              options = build_metadata_hash_from(args)
+              options.update(:pending => RSpec::Core::Pending::NOT_YET_IMPLEMENTED) unless block
+              options.update(#{extra_options.inspect})
+              examples << RSpec::Core::Example.new(self, desc, options, block)
+              examples.last
             end
-            group.metadata[:shared_group_name] = name
-            group
-          end
-        END_RUBY
+          END_RUBY
+        end
+
+        # Defines an example within a group.
+        define_example_method :example
+        # Defines an example within a group.
+        #
+        # @see example
+        define_example_method :it
+        # Defines an example within a group.
+        # This is here primarily for backward compatibility with early versions
+        # of RSpec which used `context` and `specify` instead of `describe` and
+        # `it`.
+        define_example_method :specify
+
+        # Shortcut to define an example with `:focus` => true
+        define_example_method :focus,   :focused => true, :focus => true
+        # Shortcut to define an example with `:focus` => true
+        define_example_method :focused, :focused => true, :focus => true
+        # Shortcut to define an example with `:focus` => true
+        # @see example
+        define_example_method :fit,     :focused => true, :focus => true
+
+        # Shortcut to define an example with :pending => true
+        define_example_method :pending,  :pending => true
+        # Shortcut to define an example with :pending => 'Temporarily disabled with xexample'
+        define_example_method :xexample, :pending => 'Temporarily disabled with xexample'
+        # Shortcut to define an example with :pending => 'Temporarily disabled with xit'
+        define_example_method :xit,      :pending => 'Temporarily disabled with xit'
+        # Shortcut to define an example with :pending => 'Temporarily disabled with xspecify'
+        define_example_method :xspecify, :pending => 'Temporarily disabled with xspecify'
+
+        # Works like `alias_method :name, :example` with the added benefit of
+        # assigning default metadata to the generated example.
+        #
+        # @note Use with caution. This extends the language used in your
+        #   specs, but does not add any additional documentation.  We use this
+        #   in rspec to define methods like `focus` and `xit`, but we also add
+        #   docs for those methods.
+        def alias_example_to name, extra={}
+          (class << self; self; end).define_example_method name, extra
+        end
+
+        # @private
+        # @macro [attach] define_nested_shared_group_method
+        #
+        #   @see SharedExampleGroup
+        def self.define_nested_shared_group_method(new_name, report_label=nil)
+          module_eval(<<-END_RUBY, __FILE__, __LINE__)
+            def #{new_name}(name, *args, &customization_block)
+              group = describe("#{report_label || "it should behave like"} \#{name}") do
+                find_and_eval_shared("examples", name, *args, &customization_block)
+              end
+              group.metadata[:shared_group_name] = name
+              group
+            end
+          END_RUBY
+        end
+
+        # Generates a nested example group and includes the shared content
+        # mapped to `name` in the nested group.
+        define_nested_shared_group_method :it_behaves_like, "behaves like"
+        # Generates a nested example group and includes the shared content
+        # mapped to `name` in the nested group.
+        define_nested_shared_group_method :it_should_behave_like
+
+        # Works like `alias_method :name, :it_behaves_like` with the added
+        # benefit of assigning default metadata to the generated example.
+        #
+        # @note Use with caution. This extends the language used in your
+        #   specs, but does not add any additional documentation.  We use this
+        #   in rspec to define `it_should_behave_like` (for backward
+        #   compatibility), but we also add docs for that method.
+        def alias_it_behaves_like_to name, *args, &block
+          (class << self; self; end).define_nested_shared_group_method name, *args, &block
+        end
       end
 
-      define_nested_shared_group_method :it_should_behave_like
-
-      class << self
-        alias_method :alias_it_should_behave_like_to, :define_nested_shared_group_method
-      end
-
-      alias_it_should_behave_like_to :it_behaves_like, "behaves like"
-
-      # Includes shared content declared with `name`.
+      # Includes shared content mapped to `name` directly in the group in which
+      # it is declared, as opposed to `it_behaves_like`, which creates a nested
+      # group. If given a block, that block is also eval'd in the current context.
       #
       # @see SharedExampleGroup
-      def self.include_context(name, *args)
-        block_given? ? block_not_supported("context") : find_and_eval_shared("context", name, *args)
+      def self.include_context(name, *args, &block)
+        find_and_eval_shared("context", name, *args, &block)
       end
 
-      # Includes shared content declared with `name`.
+      # Includes shared content mapped to `name` directly in the group in which
+      # it is declared, as opposed to `it_behaves_like`, which creates a nested
+      # group. If given a block, that block is also eval'd in the current context.
       #
       # @see SharedExampleGroup
-      def self.include_examples(name, *args)
-        block_given? ? block_not_supported("examples") : find_and_eval_shared("examples", name, *args)
-      end
-
-      # @private
-      def self.block_not_supported(label)
-        warn("Customization blocks not supported for include_#{label}.  Use it_behaves_like instead.")
+      def self.include_examples(name, *args, &block)
+        find_and_eval_shared("examples", name, *args, &block)
       end
 
       # @private
       def self.find_and_eval_shared(label, name, *args, &customization_block)
         raise ArgumentError, "Could not find shared #{label} #{name.inspect}" unless
-          shared_block = world.shared_example_groups[name]
+        shared_block = shared_example_groups[name]
 
         module_eval_with_args(*args, &shared_block)
         module_eval(&customization_block) if customization_block
@@ -149,7 +194,7 @@ module RSpec
       end
 
       # @private
-      # @return [Metadata] belonging to the parent of a nested [ExampleGroup](ExampleGroup)
+      # @return [Metadata] belonging to the parent of a nested {ExampleGroup}
       def self.superclass_metadata
         @superclass_metadata ||= self.superclass.respond_to?(:metadata) ? self.superclass.metadata : nil
       end
@@ -199,12 +244,19 @@ module RSpec
         subclass = Class.new(parent)
         subclass.set_it_up(*args)
         subclass.module_eval(&example_group_block) if example_group_block
+
+        # The LetDefinitions module must be included _after_ other modules
+        # to ensure that it takes precendence when there are name collisions.
+        # Thus, we delay including it until after the example group block
+        # has been eval'd.
+        MemoizedHelpers.define_helpers_on(subclass)
+
         subclass
       end
 
       # @private
       def self.children
-        @children ||= [].extend(Extensions::Ordered)
+        @children ||= [].extend(Extensions::Ordered::ExampleGroups)
       end
 
       # @private
@@ -212,9 +264,9 @@ module RSpec
         @_descendants ||= [self] + children.inject([]) {|list, c| list + c.descendants}
       end
 
-      # @private
-      def self.ancestors
-        @_ancestors ||= super().select {|a| a < RSpec::Core::ExampleGroup}
+      ## @private
+      def self.parent_groups
+        @parent_groups ||= ancestors.select {|a| a < RSpec::Core::ExampleGroup}
       end
 
       # @private
@@ -246,20 +298,8 @@ module RSpec
         args << build_metadata_hash_from(args)
         args.unshift(symbol_description) if symbol_description
         @metadata = RSpec::Core::Metadata.new(superclass_metadata).process(*args)
+        hooks.register_globals(self, RSpec.configuration.hooks)
         world.configure_group(self)
-        [:before, :after, :around].each do |_when|
-          RSpec.configuration.hooks[_when][:each].each do |hook|
-            unless ancestors.any? {|a| a.hooks[_when][:each].include? hook }
-              hooks[_when][:each] << hook # each's get filtered later per example
-            end
-          end
-          next if _when == :around # no around(:all) hooks
-          RSpec.configuration.hooks[_when][:all].each do |hook|
-            unless ancestors.any? {|a| a.hooks[_when][:all].include? hook }
-              hooks[_when][:all] << hook if hook.options_apply?(self)
-            end
-          end
-        end
       end
 
       # @private
@@ -270,6 +310,7 @@ module RSpec
       # @private
       def self.store_before_all_ivars(example_group_instance)
         return if example_group_instance.instance_variables.empty?
+
         example_group_instance.instance_variables.each { |ivar|
           before_all_ivars[ivar] = example_group_instance.instance_variable_get(ivar)
         }
@@ -283,9 +324,15 @@ module RSpec
       # @private
       def self.run_before_all_hooks(example_group_instance)
         return if descendant_filtered_examples.empty?
-        assign_before_all_ivars(superclass.before_all_ivars, example_group_instance)
-        run_hook(:before, :all, example_group_instance)
-        store_before_all_ivars(example_group_instance)
+        begin
+          assign_before_all_ivars(superclass.before_all_ivars, example_group_instance)
+
+          BeforeAllMemoizedHash.isolate_for_before_all(example_group_instance) do
+            run_hook(:before, :all, example_group_instance)
+          end
+        ensure
+          store_before_all_ivars(example_group_instance)
+        end
       end
 
       # @private
@@ -308,18 +355,7 @@ module RSpec
         return if descendant_filtered_examples.empty?
         assign_before_all_ivars(before_all_ivars, example_group_instance)
 
-        begin
-          run_hook(:after, :all, example_group_instance)
-        rescue => e
-          # TODO: come up with a better solution for this.
-          RSpec.configuration.reporter.message <<-EOS
-
-An error occurred in an after(:all) hook.
-  #{e.class}: #{e.message}
-  occurred at #{e.backtrace.first}
-
-        EOS
-        end
+        run_hook(:after, :all, example_group_instance)
       end
 
       # Runs all the examples in this group
@@ -336,6 +372,7 @@ An error occurred in an after(:all) hook.
           results_for_descendants = children.ordered.map {|child| child.run(reporter)}.all?
           result_for_this_group && results_for_descendants
         rescue Exception => ex
+          RSpec.wants_to_quit = true if fail_fast?
           fail_filtered_examples(ex, reporter)
         ensure
           run_after_all_hooks(new)
@@ -392,7 +429,7 @@ An error occurred in an after(:all) hook.
 
       # @private
       def self.top_level_description
-        ancestors.last.description
+        parent_groups.last.description
       end
 
       # @private
@@ -401,13 +438,14 @@ An error occurred in an after(:all) hook.
       end
 
       # @attr_reader
-      # Returns the [Example](Example) object that wraps this instance of
+      # Returns the {Example} object that wraps this instance of
       # `ExampleGroup`
       attr_accessor :example
 
-      # @deprecated use [example](ExampleGroup#example-instance_method)
+      # @deprecated use {ExampleGroup#example}
       def running_example
-        RSpec.deprecate("running_example", "example")
+        RSpec.deprecate("running_example",
+                        :replacement => "example")
         example
       end
 
@@ -428,12 +466,12 @@ An error occurred in an after(:all) hook.
       # @private
       # instance_evals the block, capturing and reporting an exception if
       # raised
-      def instance_eval_with_rescue(&hook)
+      def instance_eval_with_rescue(context = nil, &hook)
         begin
           instance_eval(&hook)
         rescue Exception => e
           raise unless example
-          example.set_exception(e)
+          example.set_exception(e, context)
         end
       end
     end
