@@ -7,15 +7,15 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
 
   describe "#summary_line" do
     it "with 0s outputs pluralized (excluding pending)" do
-      formatter.summary_line(0,0,0).should eq("0 examples, 0 failures")
+      expect(formatter.summary_line(0,0,0)).to eq("0 examples, 0 failures")
     end
 
     it "with 1s outputs singular (including pending)" do
-      formatter.summary_line(1,1,1).should eq("1 example, 1 failure, 1 pending")
+      expect(formatter.summary_line(1,1,1)).to eq("1 example, 1 failure, 1 pending")
     end
 
     it "with 2s outputs pluralized (including pending)" do
-      formatter.summary_line(2,2,2).should eq("2 examples, 2 failures, 2 pending")
+      expect(formatter.summary_line(2,2,2)).to eq("2 examples, 2 failures, 2 pending")
     end
   end
 
@@ -27,7 +27,7 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
       line = __LINE__ - 2
       group.run(formatter)
       formatter.dump_commands_to_rerun_failed_examples
-      output.string.should include("rspec #{RSpec::Core::Metadata::relative_path("#{__FILE__}:#{line}")} # example group fails")
+      expect(output.string).to include("rspec #{RSpec::Core::Metadata::relative_path("#{__FILE__}:#{line}")} # example group fails")
     end
   end
 
@@ -42,12 +42,12 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
     end
 
     it "preserves formatting" do
-      group.example("example name") { "this".should eq("that") }
+      group.example("example name") { expect("this").to eq("that") }
 
       run_all_and_dump_failures
 
-      output.string.should =~ /group name example name/m
-      output.string.should =~ /(\s+)expected: \"that\"\n\1     got: \"this\"/m
+      expect(output.string).to match(/group name example name/m)
+      expect(output.string).to match(/(\s+)expected: \"that\"\n\1     got: \"this\"/m)
     end
 
     context "with an exception without a message" do
@@ -55,22 +55,31 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
         exception_without_message = Exception.new()
         exception_without_message.stub(:message) { nil }
         group.example("example name") { raise exception_without_message }
-        expect { run_all_and_dump_failures }.not_to raise_error(NoMethodError)
+        expect { run_all_and_dump_failures }.not_to raise_error
       end
 
       it "preserves ancestry" do
         example = group.example("example name") { raise "something" }
         run_all_and_dump_failures
-        example.example_group.ancestors.size.should == 1
+        expect(example.example_group.parent_groups.size).to eq 1
       end
     end
 
     context "with an exception that has an exception instance as its message" do
-      it "should not raise NoMethodError" do
+      it "does not raise NoMethodError" do
         gonzo_exception = RuntimeError.new
         gonzo_exception.stub(:message) { gonzo_exception }
         group.example("example name") { raise gonzo_exception }
-        expect { run_all_and_dump_failures }.not_to raise_error(NoMethodError)
+        expect { run_all_and_dump_failures }.not_to raise_error
+      end
+    end
+
+    context "with an instance of an anonymous exception class" do
+      it "substitutes '(anonymous error class)' for the missing class name" do
+        exception = Class.new(StandardError).new
+        group.example("example name") { raise exception }
+        run_all_and_dump_failures
+        expect(output.string).to include('(anonymous error class)')
       end
     end
 
@@ -78,15 +87,15 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
       it "does not show the error class" do
         group.example("example name") { raise NameError.new('foo') }
         run_all_and_dump_failures
-        output.string.should =~ /NameError/m
+        expect(output.string).to match(/NameError/m)
       end
     end
 
     context "with a failed expectation (rspec-expectations)" do
       it "does not show the error class" do
-        group.example("example name") { "this".should eq("that") }
+        group.example("example name") { expect("this").to eq("that") }
         run_all_and_dump_failures
-        output.string.should_not =~ /RSpec/m
+        expect(output.string).not_to match(/RSpec/m)
       end
     end
 
@@ -94,15 +103,15 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
       it "does not show the error class" do
         group.example("example name") { "this".should_receive("that") }
         run_all_and_dump_failures
-        output.string.should_not =~ /RSpec/m
+        expect(output.string).not_to match(/RSpec/m)
       end
     end
 
     context 'for #share_examples_for' do
       it 'outputs the name and location' do
 
-        share_examples_for 'foo bar' do
-          it("example name") { "this".should eq("that") }
+        group.share_examples_for 'foo bar' do
+          it("example name") { expect("this").to eq("that") }
         end
 
         line = __LINE__.next
@@ -110,7 +119,7 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
 
         run_all_and_dump_failures
 
-        output.string.should include(
+        expect(output.string).to include(
           'Shared Example Group: "foo bar" called from ' +
             "./spec/rspec/core/formatters/base_text_formatter_spec.rb:#{line}"
         )
@@ -118,9 +127,9 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
 
       context 'that contains nested example groups' do
         it 'outputs the name and location' do
-          share_examples_for 'foo bar' do
+          group.share_examples_for 'foo bar' do
             describe 'nested group' do
-              it("example name") { "this".should eq("that") }
+              it("example name") { expect("this").to eq("that") }
             end
           end
 
@@ -129,7 +138,7 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
 
           run_all_and_dump_failures
 
-          output.string.should include(
+          expect(output.string).to include(
             'Shared Example Group: "foo bar" called from ' +
               "./spec/rspec/core/formatters/base_text_formatter_spec.rb:#{line}"
           )
@@ -138,10 +147,12 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
     end
 
     context 'for #share_as' do
+      before { allow(RSpec).to receive(:deprecate) }
+
       it 'outputs the name and location' do
 
-        share_as :FooBar do
-          it("example name") { "this".should eq("that") }
+        group.share_as :FooBar do
+          it("example name") { expect("this").to eq("that") }
         end
 
         line = __LINE__.next
@@ -149,7 +160,7 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
 
         run_all_and_dump_failures
 
-        output.string.should include(
+        expect(output.string).to include(
           'Shared Example Group: "FooBar" called from ' +
             "./spec/rspec/core/formatters/base_text_formatter_spec.rb:#{line}"
         )
@@ -158,10 +169,10 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
       context 'that contains nested example groups' do
         it 'outputs the name and location' do
 
-          share_as :NestedFoo do
+          group.share_as :NestedFoo do
             describe 'nested group' do
               describe 'hell' do
-                it("example name") { "this".should eq("that") }
+                it("example name") { expect("this").to eq("that") }
               end
             end
           end
@@ -171,7 +182,7 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
 
           run_all_and_dump_failures
 
-          output.string.should include(
+          expect(output.string).to include(
             'Shared Example Group: "NestedFoo" called from ' +
               "./spec/rspec/core/formatters/base_text_formatter_spec.rb:#{line}"
           )
@@ -194,12 +205,12 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
       before { RSpec.configuration.stub(:show_failures_in_pending_blocks?) { true } }
 
       it "preserves formatting" do
-        group.example("example name") { pending { "this".should eq("that") } }
+        group.example("example name") { pending { expect("this").to eq("that") } }
 
         run_all_and_dump_pending
 
-        output.string.should =~ /group name example name/m
-        output.string.should =~ /(\s+)expected: \"that\"\n\1     got: \"this\"/m
+        expect(output.string).to match(/group name example name/m)
+        expect(output.string).to match(/(\s+)expected: \"that\"\n\1     got: \"this\"/m)
       end
 
       context "with an exception without a message" do
@@ -207,7 +218,7 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
           exception_without_message = Exception.new()
           exception_without_message.stub(:message) { nil }
           group.example("example name") { pending { raise exception_without_message } }
-          expect { run_all_and_dump_pending }.not_to raise_error(NoMethodError)
+          expect { run_all_and_dump_pending }.not_to raise_error
         end
       end
 
@@ -215,15 +226,15 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
         it "does not show the error class" do
           group.example("example name") { pending { raise NameError.new('foo') } }
           run_all_and_dump_pending
-          output.string.should =~ /NameError/m
+          expect(output.string).to match(/NameError/m)
         end
       end
 
       context "with a failed expectation (rspec-expectations)" do
         it "does not show the error class" do
-          group.example("example name") { pending { "this".should eq("that") } }
+          group.example("example name") { pending { expect("this").to eq("that") } }
           run_all_and_dump_pending
-          output.string.should_not =~ /RSpec/m
+          expect(output.string).not_to match(/RSpec/m)
         end
       end
 
@@ -231,15 +242,15 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
         it "does not show the error class" do
           group.example("example name") { pending { "this".should_receive("that") } }
           run_all_and_dump_pending
-          output.string.should_not =~ /RSpec/m
+          expect(output.string).not_to match(/RSpec/m)
         end
       end
 
       context 'for #share_examples_for' do
         it 'outputs the name and location' do
 
-          share_examples_for 'foo bar' do
-            it("example name") { pending { "this".should eq("that") } }
+          group.share_examples_for 'foo bar' do
+            it("example name") { pending { expect("this").to eq("that") } }
           end
 
           line = __LINE__.next
@@ -247,7 +258,7 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
 
           run_all_and_dump_pending
 
-          output.string.should include(
+          expect(output.string).to include(
             'Shared Example Group: "foo bar" called from ' +
             "./spec/rspec/core/formatters/base_text_formatter_spec.rb:#{line}"
           )
@@ -255,9 +266,9 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
 
         context 'that contains nested example groups' do
           it 'outputs the name and location' do
-            share_examples_for 'foo bar' do
+            group.share_examples_for 'foo bar' do
               describe 'nested group' do
-                it("example name") { pending { "this".should eq("that") } }
+                it("example name") { pending { expect("this").to eq("that") } }
               end
             end
 
@@ -266,7 +277,7 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
 
             run_all_and_dump_pending
 
-            output.string.should include(
+            expect(output.string).to include(
               'Shared Example Group: "foo bar" called from ' +
               "./spec/rspec/core/formatters/base_text_formatter_spec.rb:#{line}"
             )
@@ -275,10 +286,12 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
       end
 
       context 'for #share_as' do
+        before { allow(RSpec).to receive(:deprecate) }
+
         it 'outputs the name and location' do
 
-          share_as :FooBar2 do
-            it("example name") { pending { "this".should eq("that") } }
+          group.share_as :FooBar2 do
+            it("example name") { pending { expect("this").to eq("that") } }
           end
 
           line = __LINE__.next
@@ -286,7 +299,7 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
 
           run_all_and_dump_pending
 
-          output.string.should include(
+          expect(output.string).to include(
             'Shared Example Group: "FooBar2" called from ' +
             "./spec/rspec/core/formatters/base_text_formatter_spec.rb:#{line}"
           )
@@ -295,10 +308,10 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
         context 'that contains nested example groups' do
           it 'outputs the name and location' do
 
-            share_as :NestedFoo2 do
+            group.share_as :NestedFoo2 do
               describe 'nested group' do
                 describe 'hell' do
-                  it("example name") { pending { "this".should eq("that") } }
+                  it("example name") { pending { expect("this").to eq("that") } }
                 end
               end
             end
@@ -308,7 +321,7 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
 
             run_all_and_dump_pending
 
-            output.string.should include(
+            expect(output.string).to include(
               'Shared Example Group: "NestedFoo2" called from ' +
               "./spec/rspec/core/formatters/base_text_formatter_spec.rb:#{line}"
             )
@@ -321,43 +334,161 @@ describe RSpec::Core::Formatters::BaseTextFormatter do
       before { RSpec.configuration.stub(:show_failures_in_pending_blocks?) { false } }
 
       it "does not output the failure information" do
-        group.example("example name") { pending { "this".should eq("that") } }
+        group.example("example name") { pending { expect("this").to eq("that") } }
         run_all_and_dump_pending
-        output.string.should_not =~ /(\s+)expected: \"that\"\n\1     got: \"this\"/m
+        expect(output.string).not_to match(/(\s+)expected: \"that\"\n\1     got: \"this\"/m)
       end
     end
   end
 
-  describe "#dump_profile" do
+  describe "#dump_profile_slowest_examples" do
+    example_line_number = nil
+
     before do
       group = RSpec::Core::ExampleGroup.describe("group") do
-        example("example") { sleep 0.1 }
+        # Use a sleep so there is some measurable time, to ensure
+        # the reported percent is 100%, not 0%.
+        example("example") { sleep 0.001 }
+        example_line_number = __LINE__ - 1
       end
       group.run(double('reporter').as_null_object)
 
       formatter.stub(:examples) { group.examples }
+      RSpec.configuration.stub(:profile_examples) { 10 }
     end
 
     it "names the example" do
-      formatter.dump_profile
-      output.string.should =~ /group example/m
+      formatter.dump_profile_slowest_examples
+      expect(output.string).to match(/group example/m)
     end
 
     it "prints the time" do
-      formatter.dump_profile
-      output.string.should =~ /0(\.\d+)? seconds/
+      formatter.dump_profile_slowest_examples
+      expect(output.string).to match(/0(\.\d+)? seconds/)
     end
 
     it "prints the path" do
-      formatter.dump_profile
+      formatter.dump_profile_slowest_examples
       filename = __FILE__.split(File::SEPARATOR).last
 
-      output.string.should =~ /#{filename}\:#{__LINE__ - 21}/
+      expect(output.string).to match(/#{filename}\:#{example_line_number}/)
     end
 
     it "prints the percentage taken from the total runtime" do
-      formatter.dump_profile
-      output.string.should =~ /, 100.0% of total time\):/
+      formatter.dump_profile_slowest_examples
+      expect(output.string).to match(/, 100.0% of total time\):/)
     end
   end
+
+  describe "#dump_profile_slowest_example_groups" do
+    let(:group) do 
+      RSpec::Core::ExampleGroup.describe("slow group") do
+        # Use a sleep so there is some measurable time, to ensure
+        # the reported percent is 100%, not 0%.
+        example("example") { sleep 0.01 }
+      end 
+    end
+    let(:rpt) { double('reporter').as_null_object }
+
+    before do
+      group.run(rpt)
+      RSpec.configuration.stub(:profile_examples) { 10 }
+    end
+
+    context "with one example group" do
+      before { formatter.stub(:examples) { group.examples } }
+
+      it "doesn't profile a single example group" do
+        formatter.dump_profile_slowest_example_groups
+        expect(output.string).not_to match(/slowest example groups/)
+      end
+    end
+
+    context "with multiple example groups" do
+      before do
+        group2 = RSpec::Core::ExampleGroup.describe("fast group") do
+          example("example 1") { sleep 0.004 }
+          example("example 2") { sleep 0.007 }
+        end
+        group2.run(rpt)
+
+        formatter.stub(:examples) { group.examples + group2.examples }
+      end
+
+      it "prints the slowest example groups" do
+        formatter.dump_profile_slowest_example_groups
+        expect(output.string).to match(/slowest example groups/)
+      end
+
+      it "prints the time" do
+        formatter.dump_profile_slowest_example_groups
+        expect(output.string).to match(/0(\.\d+)? seconds/)
+      end
+
+      it "ranks the example groups by average time" do
+        formatter.dump_profile_slowest_example_groups
+        expect(output.string).to match(/slow group(.*)fast group/m)
+      end
+    end
+
+    it "depends on parent_groups to get the top level example group" do
+      ex = ""
+      group.describe("group 2") do 
+        describe "group 3" do
+          ex = example("nested example 1") 
+        end
+      end
+
+      expect(ex.example_group.parent_groups.last).to eq(group)
+    end
+  end
+
+  describe "custom_colors" do
+    it "uses the custom success color" do
+      RSpec.configure do |config|
+        config.color_enabled = true
+        config.tty = true
+        config.success_color = :cyan
+      end
+      formatter.dump_summary(0,1,0,0)
+      expect(output.string).to include("\e[36m")
+    end
+  end
+
+  describe "#colorize" do
+    it "accepts a VT100 integer code and formats the text with it" do
+       expect(formatter.colorize('abc', 32)).to eq "\e[32mabc\e[0m"
+    end
+
+    it "accepts a symbol as a color parameter and translates it to the correct integer code, then formats the text with it" do
+       expect(formatter.colorize('abc', :green)).to eq "\e[32mabc\e[0m"
+    end
+
+    it "accepts a non-default color symbol as a parameter and translates it to the correct integer code, then formats the text with it" do
+       expect(formatter.colorize('abc', :cyan)).to eq "\e[36mabc\e[0m"
+    end
+  end
+
+  described_class::VT100_COLORS.each do |name, number|
+    next if name == :black
+
+    describe "##{name}" do
+      before do
+        allow(RSpec.configuration).to receive(:color_enabled?) { true }
+        allow(RSpec).to receive(:deprecate)
+      end
+
+      it "prints the text using the color code for #{name}" do
+        expect(formatter.send(name, "text")).to eq("\e[#{number}mtext\e[0m")
+      end
+
+      it "prints a deprecation warning" do
+        expect(RSpec).to receive(:deprecate) {|*args|
+          expect(args.first).to match(/#{name}/)
+        }
+        formatter.send(name, "text")
+      end
+    end
+  end
+
 end
