@@ -3,7 +3,7 @@ require 'spec_helper'
 describe "extensions" do
   describe "debugger" do
     it "is defined on Kernel" do
-      Kernel.should respond_to(:debugger)
+      expect(Kernel).to respond_to(:debugger)
     end
   end
 end
