@@ -1,3 +1,5 @@
+require 'spec_helper'
+
 module RSpec
   module Core
     describe RubyProject do
@@ -7,14 +9,14 @@ module RSpec
         context "with ancestor containing spec directory" do
           it "returns ancestor containing the spec directory" do
             RubyProject.stub(:ascend_until).and_return('foodir')
-            RubyProject.determine_root.should == 'foodir'
+            expect(RubyProject.determine_root).to eq("foodir")
           end
         end
 
         context "without ancestor containing spec directory" do
           it "returns current working directory" do
             RubyProject.stub(:find_first_parent_containing).and_return(nil)
-            RubyProject.determine_root.should == '.'
+            expect(RubyProject.determine_root).to eq(".")
           end
         end
 
